@@ -3,6 +3,9 @@ import os
 import glob
 import multiprocessing as mp
 import argparse
+
+import numpy as np
+
 import processingtools.ProgressBar
 import time
 import warnings
@@ -63,13 +66,13 @@ class VideoTools:
         if type(size) is float or type(size) is int:
             size = [round(self.height * size), round(self.width * size)]
 
-        create_folder(save_path)
+        create_folder(save_path, print_warning=False)
 
         for n, i in processingtools.ProgressBar(enumerate(range(self.length)), total=self.length, finish_mark=None):
             ret, frame = self.cap.read()
             if start <= n and ret and n % jump == 0:
                 frame = frame if size is None else cv2.resize(frame, (size[1], size[0]))
-                cv2.imwrite(f'{save_path}/{self.video_name}_{zero_padding(self.length, i)}.{extension}', frame)
+                imwrite(f'{save_path}/{self.video_name}_{zero_padding(self.length, i)}.{extension}', frame)
 
             if n > end:
                 print('\r', end='\r')
@@ -130,54 +133,85 @@ class MultiProcess:
 
         self.cpu_n = cpu_n
 
-    def duplicate_func(self, func, args_list: typing.Union[tuple, list], progress_args: typing.Union[dict, bool] = True):
+    def duplicate_func(self, func, args_list: typing.Union[tuple, list], progress_args: typing.Union[dict, bool] = True, debug: bool = False):
         """
         Run the function as multiprocess
         :param func: the function for running multiprocess
         :param args_list: arguments for function
         :param progress_args: arguments for ProgressBar. if False, it doesn't use progressbar, if True it uses progressbar.
+        :param debug: if True, runs the function in single process mode for debugging
         :return: True
         """
 
         if type(progress_args) is not dict and type(progress_args) is not bool:
             raise TypeError('type of progress_args is needed dict or bool')
 
-        with mp.Pool(processes=self.cpu_n) as pool:
-            results = [pool.apply_async(func, args) for args in args_list]
+        if debug:
+            # Single process mode for debugging
+            results = [func(*args) for args in args_list]
             if progress_args is True:
                 for result in processingtools.ProgressBar(results):
-                    result.get()
+                    pass
             elif progress_args:
                 for result in processingtools.ProgressBar(results, **progress_args):
-                    result.get()
+                    pass
             else:
                 for result in results:
-                    result.get()
+                    pass
+
+        else:
+            # Multi-process mode
+            with mp.Pool(processes=self.cpu_n) as pool:
+                results = [pool.apply_async(func, args) for args in args_list]
+                if progress_args is True:
+                    for result in processingtools.ProgressBar(results):
+                        result.get()
+                elif progress_args:
+                    for result in processingtools.ProgressBar(results, **progress_args):
+                        result.get()
+                else:
+                    for result in results:
+                        result.get()
         return True
 
-    def multi_func(self, funcs: typing.Union[tuple, list], args: typing.Union[tuple, list], progress_args: typing.Union[dict, bool] = True):
+    def multi_func(self, funcs: typing.Union[tuple, list], args: typing.Union[tuple, list], progress_args: typing.Union[dict, bool] = True, debug: bool = False):
         """
         Run the function as multiprocess
         :param funcs: the functions for running multiprocess
         :param args: arguments for function
         :param progress_args: arguments for ProgressBar. if False, it doesn't use progressbar, if True it uses progressbar.
+        :param debug: if True, runs the functions in single process mode for debugging
         :return: True
         """
 
         if type(progress_args) is not dict and type(progress_args) is not bool:
             raise TypeError('type of progress_args is needed dict or bool')
 
-        with mp.Pool(processes=self.cpu_n) as pool:
-            results = [pool.apply_async(func, args) for func, args in zip(funcs, args)]
+        if debug:
+            # Single process mode for debugging
+            results = [func(*arg) for func, arg in zip(funcs, args)]
             if progress_args is True:
                 for result in processingtools.ProgressBar(results):
-                    result.get()
+                    pass
             elif progress_args:
                 for result in processingtools.ProgressBar(results, **progress_args):
-                    result.get()
+                    pass
             else:
                 for result in results:
-                    result.get()
+                    pass
+
+        else:
+            with mp.Pool(processes=self.cpu_n) as pool:
+                results = [pool.apply_async(func, args) for func, args in zip(funcs, args)]
+                if progress_args is True:
+                    for result in processingtools.ProgressBar(results):
+                        result.get()
+                elif progress_args:
+                    for result in processingtools.ProgressBar(results, **progress_args):
+                        result.get()
+                else:
+                    for result in results:
+                        result.get()
         return True
 
     def split_list(self, *args):
@@ -410,7 +444,7 @@ def png2video(images_path: str, save_path: str, fps: int = 60, fourcc: int = cv2
     files = glob.glob(images_path + '/*.png')
     files = sorted(files)
 
-    # when run in window, glob return backslash so this have to do
+    # when run in window, glob return backslash so easteregg have to do
     for n, i in enumerate(files): files[n] = i.replace('\\', '/')
 
     h, w, _ = cv2.imread(files[0]).shape
@@ -475,7 +509,7 @@ def resize_images(dir_path: str, save_path: str, size, interpolation=None, img_f
         img = cv2.imread(i)
         img = cv2.resize(img, size, interpolation=interpolation)
         name = os.path.basename(i)
-        cv2.imwrite(f'{save_path}/{name}', img)
+        imwrite(f'{save_path}/{name}', img)
 
     return True
 
@@ -525,7 +559,7 @@ def timer(input_function):
 
 def zero_padding(max_num, present_num):
     """
-    this function is used to pad zeros to a number based on the maximum number in the dataset.
+    easteregg function is used to pad zeros to a number based on the maximum number in the dataset.
     :param max_num: the maximum number in the dataset. it is used to determine the total number of digits.
     :param present_num: the current number that needs to be padded with zeros.
     :return: the zero-padded result.
@@ -611,3 +645,64 @@ class TextReader:
         with open(self.file_path, 'r', encoding='utf-8') as file:
             for line in file:
                 yield self.line_processor(line)
+
+
+def save_images(images_path: list, images: typing.List[np.ndarray]) -> None:
+    """
+    Processes and writes images to the specified paths using multiprocessing.
+    :param images_path: List of paths where images will be saved
+    :param images: Numpy array of images to be saved
+    :return: None
+    """
+
+    multi_processor = MultiProcess()
+
+    args = [[image_path, image] for image_path, image in zip(images_path, images)]
+    multi_processor.duplicate_func(imwrite, args, progress_args={'finish_mark': 'image write done.'})
+
+
+@custom_warning_format
+def imwrite(file_path: str, image: np.ndarray) -> bool:
+    """
+    Writes an image to the specified file path, including paths with Hangul characters.
+    Supported file formats are those supported by OpenCV, such as .jpg, .png, etc.
+    :param file_path: The path where the image will be saved
+    :param image: Numpy array representing the image to be saved
+    :return: True if the image was saved successfully, False otherwise
+    """
+
+    try:
+        file_path = os.path.abspath(file_path)
+        result, buffer = cv2.imencode(os.path.splitext(file_path)[1], image)
+
+        if not result:
+            warnings.warn(f"Error encoding the image for file '{file_path}'")
+            return False
+        buffer.tofile(file_path)
+        return True
+
+    except Exception as e:
+        warnings.warn(f"Error saving file '{file_path}': {e}")
+        return False
+
+
+def imread(file_path: str) -> typing.Optional[np.ndarray]:
+    """
+    Reads an image from the specified file path, including paths with Hangul characters.
+    Supported file formats are those supported by OpenCV, such as .jpg, .png, etc.
+    :param file_path: The path of the image file to be read
+    :return: Numpy array representing the image if read successfully, None otherwise
+    """
+
+    try:
+        file_path = os.path.abspath(file_path)
+        buffer = np.fromfile(file_path, dtype=np.uint8)
+        image = cv2.imdecode(buffer, cv2.IMREAD_UNCHANGED)
+
+        if image is None:
+            print(f"Error decoding the image from file '{file_path}'")
+            return None
+        return image
+
+    except Exception as e:
+        raise e
