@@ -601,6 +601,7 @@ class AutoInputModel(torch.nn.Module):
         print(f'run on {processingtools.functions.s_text(f"{self.device}", styles=("bold",))}')
 
     @processingtools.functions.custom_warning_format
+    @torch.no_grad()
     def forward(self, inputs: typing.Union[str, list], batch_size: int = 1, num_workers: int = 0) -> typing.Union[torch.Tensor, dict]:
         """
         forward pass of the model
@@ -617,7 +618,7 @@ class AutoInputModel(torch.nn.Module):
 
         if isinstance(inputs, list):
             outputs = []
-            out_dict = {}
+            out_dict = {'results': {}}
 
             dataset = AutoInputDataset(inputs, transformer=self.transform)
             dataloader = torch.utils.data.DataLoader(dataset, batch_size=batch_size, shuffle=False, num_workers=num_workers)
@@ -630,7 +631,7 @@ class AutoInputModel(torch.nn.Module):
                 outputs.append(output)
 
                 for out, path in zip(output, paths):
-                    out_dict[path] = out
+                    out_dict['results'][path] = out
 
             print('\rInference done.')
             out_dict['total outputs'] = torch.cat(outputs, dim=0)
@@ -691,3 +692,57 @@ def gpu_env() -> None:
 
     for i in range(num_gpus):
         print(f'    • GPU {i}: {torch.cuda.get_device_name(i)}')
+
+
+class EnsembleModel(torch.nn.Module):
+    def __init__(self, models):
+        """
+        Initialize the EnsembleModel.
+        :param models: List of models to be ensembled.
+        """
+
+        super().__init__()
+        self.models = models
+
+    def forward(self, x, option: str = 'mean', weights: typing.Union[list, tuple, None] = None) -> torch.Tensor:
+        """
+        Perform ensemble predictions on the input data.
+        :param x: Input data.
+        :param option: Ensemble method ('mean' or 'WA' for weighted average).
+        :param weights: Weights for weighted average. Must be the same length as models if specified.
+        :return: Ensemble prediction.
+        """
+
+        # Check if the option is valid
+        if option not in ['mean', 'WA', 'sum', 'voting']:
+            raise ValueError("Option must be 'mean', 'WA', 'sum', or 'voting'.")
+
+        # Perform predictions for each model
+        predictions = [model(x) for model in self.models]
+
+        if option == 'mean':
+            ensemble_predictions = torch.mean(torch.stack(predictions), dim=0)
+
+        elif option == 'WA':
+            if weights is None or len(weights) != len(self.models):
+                raise ValueError("Weights must be provided and match the number of models for weighted averaging.")
+            weights = torch.tensor(weights, dtype=torch.float32)
+            weights /= weights.sum()  # Normalize weights
+
+            weighted_predictions = torch.stack(predictions) * weights[:, None, None, None]
+            ensemble_predictions = torch.sum(weighted_predictions, dim=0)
+
+        elif option == 'sum':
+            ensemble_predictions = torch.sum(torch.stack(predictions), dim=0)
+
+        elif option == 'voting':
+            voting_results = [torch.argmax(prediction, dim=1) for prediction in predictions]
+            voting_results = torch.stack(voting_results, dim=1)
+
+            # Vote by batch and return the most common class
+            ensemble_predictions = torch.mode(voting_results, dim=1).values
+
+        else:
+            ensemble_predictions = None
+
+        return ensemble_predictions
